Imports Kentico.CMS.TreeEngine
Imports Kentico.CMS.DataEngine
Imports Kentico.CMS.MetadataEngine
Imports System.IO
Imports System.Globalization
Imports Kentico.DirectoryUtilities
Imports Kentico.LicenseProvider

'''<summary>Shared class providing common functions for all pages.</summary>
Public Class Functions

    '''<summary>Constructor.</summary>
    Shared Sub New()
    End Sub


    '''<summary>Returns connection string used throughout the application.</summary>
    Public Shared Function GetConnectionString() As String
        Return ConfigurationSettings.AppSettings("CMSConnectionString")
    End Function


    '''<summary>Creates and returns a new, initialized instance of TreeProvider with current user set.</summary>
    Public Shared Function GetTreeProvider() As TreeProvider
        Dim tree As New TreeProvider

        tree.Connection.ConnectionString = GetConnectionString()        
        tree.CombineWithDefaultCulture = False        
        tree.UserID = GetUserID()
        Return tree
    End Function


    '''<summary>Returns User ID of the current user.</summary>
    Public Shared Function GetUserID() As Integer
        If Web.HttpContext.Current.Session("CMSUserID") Is Nothing And HttpContext.Current.User.Identity.IsAuthenticated = True Then
            Functions.SetCurrentUserID(Web.HttpContext.Current.User.Identity.Name)
        End If
        If Web.HttpContext.Current.Session("CMSUserID") Is Nothing Then
            Return 0
        Else
            Return CType(Web.HttpContext.Current.Session("CMSUserID"), Integer)
        End If
    End Function


    '''<summary>Returns physical path to the files (documents) directory.</summary>
    Public Shared Function GetFilesPath() As String
        Return HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSFilesFolder"))
    End Function


    '''<summary>Returns physical path to the metadata directory.</summary>
    Public Shared Function GetMetadataPath() As String
        Return HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSMetadataFolder"))
    End Function


    '''<summary>Returns true if the current user is authorized to access the given resource with required permission.</summary>
    '''<param name="resourceName">Resource name.</param>
    '''<param name="permissionName">Permission name.</param>
    Public Shared Function IsAuthorizedPerResource(ByVal resourceName As String, ByVal permissionName As String) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return treeSec.IsAuthorizedPerResource(resourceName, permissionName)
    End Function


    '''<summary>Returns true if the current user is authorized to access the given node with required permission.</summary>
    '''<param name="path">Node path.</param>
    '''<param name="pathType">Type of the provided path.</param>
    '''<param name="permission">Required permission.</param>
    '''<remarks>This method checks not only the permissions for the given node. Rather, it also checks if the user was granted with specified permission for the CMS.Content resource.</remarks>
    Public Shared Function IsAuthorizedPerNode(ByVal path As String, ByVal pathType As TreePathTypeEnum, ByVal permission As NodePermissionsEnum) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return treeSec.IsAuthorizedPerNode(path, pathType, permission)
    End Function


    '''<summary>Returns true if the current user is authorized to access the given class with required permission.</summary>
    '''<param name="className">Class name in format application.class.</param>    
    '''<param name="permissionName">Name of the required permission..</param>
    '''<remarks>This method checks not only the permissions for the given node. Rather, it also checks if the user was granted with specified permission for the CMS.Content resource.</remarks>
    Public Shared Function IsAuthorizedPerClass(ByVal className As String, ByVal permissionName As String) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return treeSec.IsAuthorizedPerClass(className, permissionName)
    End Function


    '''<summary>Sets current user's ID into a session variable for further use.</summary>
    '''<param name="userName">User name.</param>
    Public Shared Sub SetCurrentUserID(ByVal userName As String)
        If userName & "" = "" Then
            Exit Sub
        End If

        Dim cn As New GeneralConnection(Functions.GetConnectionString)
        Dim params(0, 2) As Object
        Dim ds As DataSet

        params(0, 0) = "@UserName"
        params(0, 1) = userName
        ds = cn.ExecuteQuery("CMS.User.selectbyusername", params)
        If ds.Tables(0).Rows.Count > 0 Then
            Web.HttpContext.Current.Session("CMSUserID") = ds.Tables(0).Rows(0)("UserID")
        Else
            Exit Sub
        End If

        'set preferred culture session
        Functions.GetPreferredUICulture()
        Functions.GetPreferredContentCultureCode()
    End Sub


    '''<summary>Redirects user to the "Access Denied" page.</summary>
    '''<param name="resourceName">Name of the resource that cannot be accessed.</param>
    '''<param name="permissionName">Name of the permission that is not allowed.</param>
    Public Shared Sub RedirectToAccessDenied(ByVal resourceName As String, ByVal permissionName As String)
        Dim basePath As String = HttpContext.Current.Request.ApplicationPath
        If basePath.EndsWith("/") Then
            basePath = basePath.Substring(0, basePath.Length - 1)
        End If
        Web.HttpContext.Current.Response.Redirect(basePath & "/cmsdesk/accessdenied.aspx?resource=" & resourceName & "&permission=" & permissionName)
    End Sub


    '''<summary>Checks if all folders of the given path exist and if not, it creates them.</summary>
    '''<param name="path">Full disk path of the file including file name.</param>
    '''<param name="startingPath">Starting path where folders can be checked and created.</param>
    Public Shared Sub EnsureDiskPath(ByVal path As String, ByVal startingPath As String)
        DirectoryHelper.EnsureDiskPath(path, startingPath)
    End Sub


    '''<summary>Creates folders and files necessary for friendly URL engine.</summary>
    '''<param name="newAliasPath">Alias path of the newly created document.</param>
    Public Shared Sub EnsureFriendlyURL(ByVal newAliasPath As String)        
        EnsureFriendlyURL(newAliasPath, "")
    End Sub


    '''<summary>Moves folders and files necessary for friendly URL engine.</summary>
    '''<param name="newAliasPath">Alias path of the new location parent node.</param>
    '''<param name="oldAliasPath">Alias path of the old document.</param>
    Public Shared Sub EnsureFriendlyURL(ByVal newAliasPath As String, ByVal oldAliasPath As String)
        If Not GenerateFriendlyUrlFiles() Then
            Exit Sub
        End If

        Dim originalNewAliasPath As String = newAliasPath

        If oldAliasPath Is Nothing Then
            oldAliasPath = ""
        End If
        If Not (newAliasPath.StartsWith(GetCMSWebSiteBaseAliasPath) And newAliasPath.Length > GetCMSWebSiteBaseAliasPath.Length) Then
            Exit Sub
        End If
        If oldAliasPath & "" <> "" AndAlso Not (oldAliasPath.StartsWith(GetCMSWebSiteBaseAliasPath) And oldAliasPath.Length > GetCMSWebSiteBaseAliasPath.Length) Then
            Exit Sub
        End If
        If Not IsUsingFriendlyURLs(newAliasPath) Then
            Exit Sub
        End If
        'adjust alias paths
        newAliasPath = newAliasPath.Substring(GetCMSWebSiteBaseAliasPath.Length)
        If oldAliasPath & "" <> "" Then
            oldAliasPath = oldAliasPath.Substring(GetCMSWebSiteBaseAliasPath.Length)
        End If
        If Functions.UseMultilingualSupport And newAliasPath.Split("/"c).Length > 2 Then
            newAliasPath = Functions.RemoveCultureFromAliasPath(newAliasPath)
            If oldAliasPath.Split("/"c).Length > 2 Then
                oldAliasPath = Functions.RemoveCultureFromAliasPath(oldAliasPath)
            End If
        End If
        'check if the new folder should be created or moved from somewhere else
        If oldAliasPath & "" <> "" Then
            'rename or move existing folder            
            Dim oldDiskPath As String = HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath) & Replace(oldAliasPath, "/", "\")
            Dim newDiskPath As String = HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath) & Replace(newAliasPath, "/", "\")
            If Directory.Exists(oldDiskPath) Then
                Functions.EnsureDiskPath(newDiskPath, HttpContext.Current.Server.MapPath(Functions.GetWebApplicationVirtualPath))
                If oldDiskPath <> newDiskPath Then
                    Try
                        DirectoryHelper.MoveDirectory(oldDiskPath, newDiskPath)
                    Catch ex As Exception
                        DirectoryHelper.CopyDirectory(oldDiskPath, newDiskPath)
                    End Try
                End If
            Else
                EnsureFriendlyURL(originalNewAliasPath)
            End If
            'move attachments folder
            If Directory.Exists(Functions.GetFileFolderPath(oldAliasPath & "/NOTHING")) Then
                Functions.EnsureDiskPath(Functions.GetFileFolderPath(newAliasPath & "/NOTHING"), HttpContext.Current.Server.MapPath(Functions.GetWebApplicationVirtualPath))
                If oldAliasPath <> newAliasPath Then
                    Try
                        DirectoryHelper.MoveDirectory(Functions.GetFileFolderPath(oldAliasPath & "/NOTHING"), Functions.GetFileFolderPath(newAliasPath & "/NOTHING"))
                    Catch ex As Exception
                        DirectoryHelper.CopyDirectory(Functions.GetFileFolderPath(oldAliasPath & "/NOTHING"), Functions.GetFileFolderPath(newAliasPath & "/NOTHING"))
                    End Try
                End If
            End If
        End If
        Try
            Dim diskPath As String = HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath) & Replace(newAliasPath, "/", "\") & "\default.aspx"
            EnsureDiskPath(diskPath, HttpContext.Current.Server.MapPath(Functions.GetWebApplicationVirtualPath))
            If File.Exists(diskPath) = False Then
                'Create a file to write to
                Dim sw As StreamWriter = File.CreateText(diskPath)
                sw.WriteLine("This is an empty ASPX file. It's used by the URL rewriting engine. Please do not remove it.")
                sw.Flush()
                sw.Close()
            End If
        Catch
        End Try
    End Sub


    '''<summary>Deletes folders and files used for friendly URLs.</summary>
    '''<param name="aliasPath">Alias path of the node being deleted.</param>
    Public Shared Sub DeleteFriendlyURL(ByVal aliasPath As String)
        If Not GenerateFriendlyUrlFiles() Then
            Exit Sub
        End If
        If Not (aliasPath.StartsWith(GetCMSWebSiteBaseAliasPath()) And aliasPath.Length > GetCMSWebSiteBaseAliasPath.Length) Then
            Exit Sub
        End If
        'adjust alias path
        aliasPath = aliasPath.Substring(GetCMSWebSiteBaseAliasPath.Length)
        If Functions.UseMultilingualSupport And aliasPath.Split("/"c).Length > 2 Then
            'delete friendly URL only in case that the default culture version is being deleted
            If Trim(LCase(aliasPath.Split("/"c)(1) & "")) <> Trim(LCase(ConfigurationSettings.AppSettings("CMSDefaultCultureCode") & "")) Then
                Exit Sub
            Else
                aliasPath = Functions.RemoveCultureFromAliasPath(aliasPath)
            End If
        End If
        Dim folderPath As String = HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath) & Replace(aliasPath, "/", "\")
        If Directory.Exists(folderPath) Then
            Try
                Directory.Delete(folderPath, True)
            Catch
            End Try
        End If
        'delete the folder in the attachments folder
        If Directory.Exists(Functions.GetAttachmentBaseDiskPath & Replace(aliasPath, "/", "\")) Then
            Try
                Directory.Delete(Functions.GetAttachmentBaseDiskPath & Replace(aliasPath, "/", "\"), True)
            Catch
            End Try
        End If
    End Sub


    '''<summary>Copies existing folders and files for friendly URLs.</summary>
    '''<param name="sourceAliasPath">Source alias path of the node that is being copied.</param>
    '''<param name="targetAliasPath">Target alias path of the parent node.</param>
    Public Shared Sub CopyFriendlyURL(ByVal sourceAliasPath As String, ByVal targetAliasPath As String)
        If Not GenerateFriendlyUrlFiles() Then
            Exit Sub
        End If
        If Not (sourceAliasPath.StartsWith(GetCMSWebSiteBaseAliasPath) And sourceAliasPath.Length > GetCMSWebSiteBaseAliasPath.Length And targetAliasPath.StartsWith(GetCMSWebSiteBaseAliasPath) And targetAliasPath.Length > GetCMSWebSiteBaseAliasPath.Length) Then
            Exit Sub
        End If
        If Not IsUsingFriendlyURLs(sourceAliasPath) Then
            Exit Sub
        End If
        'adjust alias paths
        sourceAliasPath = sourceAliasPath.Substring(GetCMSWebSiteBaseAliasPath.Length)
        targetAliasPath = targetAliasPath.Substring(GetCMSWebSiteBaseAliasPath.Length)
        Dim sourceAliasPathWithoutCulture As String
        Dim targetAliasPathWithoutCulture As String
        If Functions.UseMultilingualSupport And sourceAliasPath.Split("/"c).Length > 2 And targetAliasPath.Split("/"c).Length > 2 Then
            sourceAliasPathWithoutCulture = Functions.RemoveCultureFromAliasPath(sourceAliasPath)
            targetAliasPathWithoutCulture = Functions.RemoveCultureFromAliasPath(targetAliasPath)
        Else
            sourceAliasPathWithoutCulture = sourceAliasPath
            targetAliasPathWithoutCulture = targetAliasPath
        End If
        Dim sourceFolderPath As String = HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath) & Replace(sourceAliasPathWithoutCulture, "/", "\")
        Dim targetFolderPath As String = HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath) & Replace(targetAliasPathWithoutCulture, "/", "\")
        Dim nodeAlias As String = sourceFolderPath.Substring(sourceFolderPath.LastIndexOf("\") + 1)
        If Directory.Exists(sourceFolderPath) Then
            Try
                EnsureDiskPath(targetFolderPath, HttpContext.Current.Server.MapPath(Functions.GetWebApplicationVirtualPath))
                CopyFolder(sourceFolderPath, targetFolderPath)
            Catch ex As Exception
            End Try
        End If
        'copy folders in the Attachments/files folder if applicable
        If Directory.Exists(Functions.GetFileFolderPath(sourceAliasPath & "\NOTHING")) Then
            EnsureDiskPath(Functions.GetFileFolderPath(targetAliasPath) & "\nothing.txt", Functions.GetAttachmentBaseDiskPath)
            CopyFolder(Functions.GetFileFolderPath(sourceAliasPath & "/NOTHING"), Functions.GetFileFolderPath(targetAliasPath & "/NOTHING"))
        End If
    End Sub

    '''<summary>Moves file (attachement).</summary>
    '''<param name="sourceAliasPath">Source alias path.</param>
    '''<param name="targetAliasPath">Target alias path.</param>
    Public Shared Sub CopyAttachment(ByVal sourceAliasPath As String, ByVal targetAliasPath As String)
        Dim tree As TreeProvider = Functions.GetTreeProvider
        Dim node As TreeNode = tree.SelectSingleNode(sourceAliasPath, TreePathTypeEnum.AliasPath)
        If node.ClassName.ToLower <> "cms.file" Then
            Exit Sub
        End If
        Dim fileName As String = CType(node.GetValue("FileName"), String)
        Dim sourceFolderPath As String = Functions.GetFileFolderPath(sourceAliasPath)
        Dim targetFolderPath As String = Functions.GetFileFolderPath(targetAliasPath)
        If Not sourceFolderPath.EndsWith("\") Then
            sourceFolderPath &= "\"
        End If
        If Not targetFolderPath.EndsWith("\") Then
            targetFolderPath &= "\"
        End If
        Functions.EnsureDiskPath(targetFolderPath, Functions.GetAttachmentBaseDiskPath)
        File.Copy(sourceFolderPath & fileName, targetFolderPath & fileName, True)
    End Sub


    '''<summary>Moves file (attachement).</summary>
    '''<param name="oldAliasPath">Old alias path.</param>
    '''<param name="newAliasPath">New alias path.</param>
    Public Shared Sub MoveAttachment(ByVal oldAliasPath As String, ByVal newAliasPath As String)
        Dim tree As TreeProvider = Functions.GetTreeProvider
        Dim node As TreeNode = tree.SelectSingleNode(newAliasPath, TreePathTypeEnum.AliasPath)
        If node.ClassName.ToLower <> "cms.file" Then
            Exit Sub
        End If
        Dim fileName As String = CType(node.GetValue("FileName"), String)
        Dim sourceFolderPath As String = Functions.GetFileFolderPath(oldAliasPath)
        Dim targetFolderPath As String = Functions.GetFileFolderPath(newAliasPath)
        If Not sourceFolderPath.EndsWith("\") Then
            sourceFolderPath &= "\"
        End If
        If Not targetFolderPath.EndsWith("\") Then
            targetFolderPath &= "\"
        End If
        Functions.EnsureDiskPath(targetFolderPath, Functions.GetAttachmentBaseDiskPath)
        File.Move(sourceFolderPath & fileName, targetFolderPath & fileName)
    End Sub


    '''<summary>Returns Web application virtual path.</summary>
    '''<remarks>It uses the CMSWebApplicationVirtualPath configuration parameter. If it's not used, "~/" is returned.</remarks>
    Public Shared Function GetWebApplicationVirtualPath() As String
        If ConfigurationSettings.AppSettings("CMSWebApplicationVirtualPath") Is Nothing OrElse ConfigurationSettings.AppSettings("CMSWebApplicationVirtualPath") = "" Then
            Return "~/."
        Else
            Return ConfigurationSettings.AppSettings("CMSWebApplicationVirtualPath")
        End If
    End Function


    '''<summary>Returns web site base alias path from the application configuration.</summary>
    Public Shared Function GetCMSWebSiteBaseAliasPath() As String
        Return ConfigurationSettings.AppSettings("CMSWebSiteBaseAliasPath") & ""
    End Function


    '''<summary>Copies folder including subfolders and all files.</summary>
    '''<param name="sourceFolder">Source folder physical path.</param>
    '''<param name="targetFolder">Target parent folder physical path.</param>
    Private Shared Sub CopyFolder(ByVal sourceFolder As String, ByVal targetFolder As String)
        DirectoryHelper.CopyDirectory(sourceFolder, targetFolder)
    End Sub


    '''<summary>Returns true if node with provided aliasPath uses friendly URLs.</summary>
    '''<param name="aliasPath">Alias path of the document.</param>
    Public Shared Function IsUsingFriendlyURLs(ByVal aliasPath As String) As Boolean
        Dim tree As TreeProvider = GetTreeProvider()
        Dim node As TreeNode = tree.SelectSingleNode(aliasPath, TreePathTypeEnum.AliasPath)
        If node Is Nothing Then
            Return False
        Else
            If MetadataProvider.GetSettingsValue(node.ClassName, "SupportFriendlyURLs") = "true" Then
                Return True
            Else
                Return False
            End If
        End If
    End Function


    '''<summary>Returns true if multilingual support is switched on.</summary>
    Public Shared Function UseMultilingualSupport() As Boolean
        If Trim(LCase(ConfigurationSettings.AppSettings("CMSUseMultilingualSupport") & "")) = "true" Then
            Return True
        Else
            Return False
        End If
    End Function

    '''<summary>Removes culture from alias path if you're using multilingual support with hidden culture.</summary>
    Public Shared Function RemoveCultureFromAliasPath(ByVal aliasPath As String) As String
        If UseMultilingualSupport() Then
            Return aliasPath.Substring(aliasPath.Split("/"c)(1).Length + 1)
        Else
            Return aliasPath
        End If
    End Function


    '''<summary>Returns alias path of the node with required culture.</summary>
    Public Shared Function GetMultilingualAliasPath(ByVal idpath As String, ByVal requiredCulture As String) As String
        'get node
        Dim node As TreeNode
        Dim tree As TreeProvider = Functions.GetTreeProvider
        Dim cultureNode As TreeNode

        'first check if document is placed under some cultureNode or if it is a culture node
        If idpath.Split("/"c)(1) = "" Then
            Return "/"
        Else
            node = tree.SelectSingleNode(idpath, TreePathTypeEnum.IDPath)
            cultureNode = tree.SelectSingleNode("/" & idpath.Split("/"c)(1), TreePathTypeEnum.IDPath)
            If cultureNode.ClassName.ToLower = "cms.culture" Then
                'this is culture node - return alias path with required culture
                Return "/" & requiredCulture & Functions.RemoveCultureFromAliasPath(CType(node.GetValue("AliasPath"), String))
            Else
                'this document is placed outside the culture zone - return the actual alias path
                Return CType(node.GetValue("AliasPath"), String)
            End If
        End If
    End Function


    '''<summary>Returns default culture for multilingual support.</summary>
    Public Shared Function GetDefaultCulture() As String
        Return ConfigurationSettings.AppSettings("CMSDefaultCultureCode") & ""
    End Function


    '''<summary>Returns preferred UI culture of the current user.</summary>
    Public Shared Function GetPreferredUICulture() As CultureInfo
        If Functions.GetUserID = 0 Then
            If GetDefaultCulture() = "" Then
                HttpContext.Current.Session("CMSUserPreferredUICulture") = System.Threading.Thread.CurrentThread.CurrentCulture
            Else
                HttpContext.Current.Session("CMSUserPreferredUICulture") = New CultureInfo(GetDefaultCulture)
            End If
        End If
        If HttpContext.Current.Session("CMSUserPreferredUICulture") Is Nothing Then
            'get preferred language
            Dim userObj As DataClass
            Dim tree As TreeProvider = Functions.GetTreeProvider

            userObj = New DataClass("cms.user", Functions.GetUserID, tree.Connection)
            If Trim(CType(userObj.GetValue("PreferredUICultureCode"), String) & "") <> "" Then
                HttpContext.Current.Session("CMSUserPreferredUICulture") = New CultureInfo(Trim(CType(userObj.GetValue("PreferredUICultureCode"), String) & ""))
            Else
                HttpContext.Current.Session("CMSUserPreferredUICulture") = System.Threading.Thread.CurrentThread.CurrentCulture
            End If
        End If
        System.Threading.Thread.CurrentThread.CurrentCulture = CType(HttpContext.Current.Session("CMSUserPreferredUICulture"), CultureInfo)
        System.Threading.Thread.CurrentThread.CurrentUICulture = CType(HttpContext.Current.Session("CMSUserPreferredUICulture"), CultureInfo)
        Return CType(HttpContext.Current.Session("CMSUserPreferredUICulture"), CultureInfo)
    End Function


    '''<summary>Returns the code of the preferred content culture of the current user.</summary>
    Public Shared Function GetPreferredContentCultureCode() As String
        If Functions.GetUserID = 0 Then
            Exit Function
        End If
        If HttpContext.Current.Session("CMSPreferredCulture") Is Nothing Then
            HttpContext.Current.Session("CMSPreferredCulture") = ConfigurationSettings.AppSettings("CMSDefaultCultureCode") & ""
            HttpContext.Current.Response.Cookies.Add(New HttpCookie("CMSPreferredCulture", ConfigurationSettings.AppSettings("CMSDefaultCultureCode") & ""))
            'get preferred language
            Dim userObj As DataClass
            Dim tree As TreeProvider = Functions.GetTreeProvider

            userObj = New DataClass("cms.user", Functions.GetUserID, tree.Connection)
            If Trim(CType(userObj.GetValue("PreferredContentCultureCode"), String) & "") <> "" Then
                HttpContext.Current.Session("CMSPreferredCulture") = Trim(CType(userObj.GetValue("PreferredContentCultureCode"), String) & "")
                HttpContext.Current.Response.Cookies.Add(New HttpCookie("CMSPreferredCulture", Trim(CType(userObj.GetValue("PreferredContentCultureCode"), String) & "")))
            End If
        End If
        Return CType(HttpContext.Current.Session("CMSPreferredCulture"), String)
    End Function


    '''<summary>Returns starting IDPath from the user profile of the current user. If no path is specified, root node ("/") is returned.</summary>
    Public Shared Function GetUsersStartingPath() As String
        Dim idpath As String = "/"
        'try to get starting path from user profile
        Dim tree As TreeProvider
        Dim userObj As DataClass
        Dim node As TreeNode

        tree = Functions.GetTreeProvider
        userObj = New DataClass("cms.user", Functions.GetUserID, tree.Connection)
        If userObj.IsEmpty Then
            Return "/"
        End If
        If Trim(CType(userObj.GetValue("StartingAliasPath"), String) & "") <> "" Then
            node = tree.SelectSingleNode(Trim(CType(userObj.GetValue("StartingAliasPath"), String) & ""), TreePathTypeEnum.AliasPath)
            If Not node Is Nothing Then
                idpath = CType(node.GetValue("idpath"), String)
            Else
                Throw New ApplicationException("Invalid starting path.")
            End If
        End If

        Return idpath
    End Function


    '''<summary>Returns starting AliasPath from the user profile of the current user. If no path is specified, root node ("/") is returned.</summary>
    Public Shared Function GetUsersStartingAliasPath() As String
        Dim aliaspath As String = ""

        'try to get starting path from user profile
        Dim tree As TreeProvider
        Dim userObj As DataClass
        Dim node As TreeNode

        tree = Functions.GetTreeProvider
        userObj = New DataClass("cms.user", Functions.GetUserID, tree.Connection)
        If Not userObj.IsEmpty Then
            aliaspath = Trim(CType(userObj.GetValue("StartingAliasPath"), String) & "")
        End If
        If aliaspath = "" Then
            aliaspath = "/"
        End If

        Return aliaspath
    End Function


    '''<summary>Returns node of the specified IDPath and culture.</summary>
    '''<param name="idPath">IDPath of the default culture version of the node.</param>
    '''<param name="cultureCode">Culture code of the required culture version.</param>
    '''<remarks>If node is not available for specified culture, it returns nothing (null). If multilingual support is not used or no culture code is provided, it returns the node with specified idpath.</remarks>
    Public Shared Function GetNodeByIDPathAndCulture(ByVal idPath As String, ByVal cultureCode As String) As TreeNode
        Dim tree As TreeProvider
        Dim node As TreeNode
        Dim aliaspath As String

        tree = Functions.GetTreeProvider
        node = tree.SelectSingleNode(idPath, TreePathTypeEnum.IDPath)
        If Functions.UseMultilingualSupport And cultureCode & "" <> "" Then
            aliaspath = CType(node.GetValue("AliasPath"), String)
            node = tree.SelectSingleNode("/" & cultureCode & Functions.RemoveCultureFromAliasPath(aliaspath), TreePathTypeEnum.AliasPath)
        End If
        Return node
    End Function


    '''<summary>Returns true if current installation of Kentico CMS is an Express Edition.</summary>
    '''<param name="featureName">Name of the requested feature.</param>
    Public Shared Function RedirectExpressEdition(ByVal featureName As String) As Boolean
        If LicenseManager.GetProductCode = "CX" Then
            Dim basePath As String = HttpContext.Current.Request.ApplicationPath
            If basePath.EndsWith("/") Then
                basePath = basePath.Substring(0, basePath.Length - 1)
            End If
            Web.HttpContext.Current.Response.Redirect(basePath & "/cmsdesk/editionerror.aspx?feature=" & featureName)
        End If
    End Function


    '''<summary>Returns required value from the class definition.</summary>
    '''<param name="classId">Class ID.</param>
    '''<param name="columnName">Column name whose value should be returned.</param>
    Public Shared Function GetClassDRValue(ByVal classId As Integer, ByVal columnName As String) As Object
        Return ClassInformationProvider.GetClassRow(classId, GetTreeProvider)(columnName)
    End Function


    '''<summary>Deletes attachment.</summary>
    '''<param name="aliasPath">Alias path of the attachment.</param>
    Public Shared Sub DeleteAttachment(ByVal aliasPath As String)
        Dim tree As TreeProvider = GetTreeProvider()
        Dim node As TreeNode
        Dim diskPath As String

        node = tree.SelectSingleNode(aliasPath, TreePathTypeEnum.AliasPath)
        If Not node Is Nothing Then
            If node.ClassName.ToLower = "cms.file" Then
                diskPath = GetFileFolderPath(aliasPath)
                If Not diskPath.EndsWith("\") Then
                    diskPath &= "\"
                End If
                diskPath = diskPath & CType(node.GetValue("FileName"), String)
                If File.Exists(diskPath) Then
                    Try
                        File.Delete(diskPath)
                    Catch ex As Exception
                    End Try
                End If
            End If
        End If
    End Sub

    '''<summary>Deletes attachment specified by its alias path and file name. This method is used when deleted file is destroyed.</summary>
    '''<param name="aliasPath">Alias path of the attachment.</param>
    '''<param name="fileName">File name of the attachment.</param>
    Public Shared Sub DeleteAttachment(ByVal aliasPath As String, ByVal fileName As String)
        Dim diskPath As String

        diskPath = GetFileFolderPath(aliasPath)
        If Not diskPath.EndsWith("\") Then
            diskPath &= "\"
        End If
        diskPath = diskPath & fileName
        If File.Exists(diskPath) Then
            Try
                File.Delete(diskPath)
            Catch ex As Exception
            End Try
        End If
    End Sub


    '''<summary>Returns base disk path of the attachments folder.</summary>
    Public Shared Function GetAttachmentBaseDiskPath() As String
        If Not ConfigurationSettings.AppSettings("CMSAttachmentsFolder") Is Nothing AndAlso ConfigurationSettings.AppSettings("CMSAttachmentsFolder") <> "" Then
            'attachments folder is specified
            If ConfigurationSettings.AppSettings("CMSAttachmentsFolder").StartsWith("/") Then
                'it's a virtual folder
                Return HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSAttachmentsFolder"))
            Else
                'it's a physical path
                Return ConfigurationSettings.AppSettings("CMSAttachmentsFolder")
            End If
        Else
            'attachments folder is not specified - use web path instead
            If GetWebApplicationVirtualPath() <> "" Then
                Return HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath)
            Else
                Return HttpContext.Current.Server.MapPath("~/.")
            End If
        End If
    End Function


    '''<summary>Returns folder path of the file specified by alias path.</summary>
    '''<param name="aliasPath">Alias Path of the node.</param>
    Public Shared Function GetFileFolderPath(ByVal aliasPath As String) As String
        Dim result As String = ""

        aliasPath = Functions.RemoveCultureFromAliasPath(aliasPath)
        If aliasPath.ToLower.StartsWith("/files") Then
            'this is a files folder
            If (Not ConfigurationSettings.AppSettings("CMSFilesFolder") Is Nothing) AndAlso (ConfigurationSettings.AppSettings("CMSFilesFolder") <> "") Then
                result = Web.HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSFilesFolder"))
            Else
                result = Web.HttpContext.Current.Server.MapPath("~/files")
            End If
            'replace "/files"
            aliasPath = aliasPath.Substring("/files".Length)
        Else
            result = GetAttachmentBaseDiskPath()
        End If
        If result.EndsWith("\") Then
            result = result.Substring(0, result.Length - 1)
        End If
        'remove last part of the alias path (the file name)
        Dim pathSplit As String() = aliasPath.Split("/"c)
        aliasPath = aliasPath.Substring(0, aliasPath.Length - pathSplit(pathSplit.Length - 1).Length)
        If aliasPath.EndsWith("/") Then
            aliasPath = aliasPath.Substring(0, aliasPath.Length - 1)
        End If
        'add alias path
        result &= aliasPath.Replace("/"c, "\"c)
        Return result
    End Function


    '''<summary>Tries to create a new file and and a new directory and returns error message if it fails.</summary>
    Public Shared Function CheckSystemPermissions() As String
        Dim errorMessage As String = ""

        'check file system permissions        
        Try
            Dim path As String = HttpContext.Current.Server.MapPath("~/testsystemsettings_" & HttpContext.Current.Session.SessionID & ".txt")
            Dim sw As StreamWriter = File.CreateText(path)
            sw.WriteLine("Test of permissions.")
            sw.Flush()
            sw.Close()
            File.Delete(path)
            path = HttpContext.Current.Server.MapPath("~/testsystemsettings_" & HttpContext.Current.Session.SessionID)
            DirectoryHelper.CreateDirectory(path)
            Directory.Delete(path)
        Catch ex As Exception
            errorMessage &= Replace(ResMan.GetString("CheckSystemPermissions.ErrorDiskAccess"), "#ACCOUNT#", System.Security.Principal.WindowsIdentity.GetCurrent().Name) & " " & HttpContext.Current.Server.MapPath("~/.")
        End Try
        Return errorMessage
    End Function


    '''<summary>Returns true if helper files for friendly URLs support should be generated in the file system.</summary>
    Public Shared Function GenerateFriendlyUrlFiles() As Boolean
        If (Not ConfigurationSettings.AppSettings("CMSGenerateFriendlyUrlFiles") Is Nothing) AndAlso ConfigurationSettings.AppSettings("CMSGenerateFriendlyUrlFiles").Trim.ToLower = "false" Then
            Return False
        Else
            Return True
        End If
    End Function


    '''<summary>Adds the postfix of the generated URLs according to the CMSFriendlyURLStyle web.config parameter value.</summary>
    '''<param name="aliasPath">Alias path that should be (optionally) converted.</param>
    Public Shared Function GetUrlForAliasPath(ByVal aliasPath As String) As String
        If (Not ConfigurationSettings.AppSettings("CMSFriendlyURLStyle") Is Nothing) AndAlso ConfigurationSettings.AppSettings("CMSFriendlyURLStyle").Trim.ToLower <> "" Then
            If aliasPath = "/" Or aliasPath = "~" Or aliasPath = "~/" Or aliasPath = "~/." Then
                Return aliasPath
            Else
                Return aliasPath & ConfigurationSettings.AppSettings("CMSFriendlyURLStyle").Trim.ToLower
            End If
        Else
            Return aliasPath
        End If
    End Function


    '''<summary>Returns configuration key value. If the configuration key value is not specified, it returns "".</summary>
    '''<param name="key">Configuration key name.</param>
    Public Shared Function GetConfigValue(ByVal key As String) As String
        If ConfigurationSettings.AppSettings(key) Is Nothing Then
            Return ""
        Else
            Return ConfigurationSettings.AppSettings(key).Trim
        End If
    End Function


    '''<summary>Writes event to the event log.</summary>
    '''<param name="eventType">Type of the event. I = information, E = error, W = warning</param>
    '''<param name="source">Source of the event (Content, Administration, etc.)</param>
    '''<param name="eventCode">Event code (Security, Update, Delete, etc.)</param>
    '''<param name="nodeId">ID value of the document.</param>
    '''<param name="nodeNamePath">NamePath value of the document.</param>
    '''<param name="eventDescription">Detailed description of the event.</param>
    Public Shared Sub LogEvent(ByVal eventType As String, ByVal source As String, ByVal eventCode As String, ByVal nodeId As Integer, ByVal nodeNamePath As String, ByVal eventDescription As String)
        Dim tree As TreeProvider = Functions.GetTreeProvider
        Dim log As New Kentico.CMS.EventLog.EventLogProvider(tree.Connection)
        log.LogEvent(eventType, Now, source, eventCode, tree.UserID, HttpContext.Current.User.Identity.Name, nodeId, nodeNamePath, HttpContext.Current.Request.UserHostAddress, eventDescription)
    End Sub

    '''<summary>Gets result of Request.QueryString</summary>
    '''<param name="key">Name of the requested key</param>
    Public Shared Function RequestQueryString(ByVal key As String) As String
        Return HttpContext.Current.Request.QueryString(key)
    End Function

    '''<summary>Returns parentPath.</summary>
    '''<param name="path">Node path (could be alias-, name- or id-path)</param>
    Public Shared Function GetParentPath(ByVal path As String) As String
        If path = "/" Then
            Return "/"
        Else
            Return path.Substring(0, path.LastIndexOf("/"))
        End If
    End Function

    '''<summary>Recursive creates parent nodes of specific culture version, if they do not exist</summary>
    '''<param name="aliaspath">Node specified by this aliasPath is created when does not exist</param>
    Public Shared Function CreateNodePath(ByVal aliaspath As String) As TreeNode
        Dim tree As TreeProvider = Functions.GetTreeProvider
        Dim node As TreeNode = tree.SelectSingleNode(aliaspath, TreePathTypeEnum.AliasPath)
        If node Is Nothing Then
            Dim parentAliasPath As String = GetParentPath(aliaspath)
            CreateNodePath(parentAliasPath)
            'copy default culture version of the specified node
            Dim defaultCulture As String = Functions.GetConfigValue("CMSDefaultCultureCode")
            Dim defaultCultureAliasPath As String = "/" & defaultCulture & Functions.RemoveCultureFromAliasPath(aliaspath)
            Dim resultNode As TreeNode = tree.CopyNode(defaultCultureAliasPath, parentAliasPath, TreePathTypeEnum.AliasPath, False)
            If (Not ConfigurationSettings.AppSettings("CMSCopyAclOfTranslatedParentDocuments") Is Nothing) AndAlso ConfigurationSettings.AppSettings("CMSCopyAclOfTranslatedParentDocuments").ToLower.Trim = "true" Then
                Dim defaultNode As TreeNode = tree.SelectSingleNode(defaultCultureAliasPath, Kentico.CMS.TreeEngine.TreePathTypeEnum.AliasPath)
                resultNode.SetValue("ACLID", defaultNode.GetValue("ACLID"))
                resultNode.Update()
            End If
            Return resultNode
        Else
            Return node
        End If
    End Function
End Class
