Imports Kentico.CMS.TreeEngine
Imports Kentico.CMS.DataEngine
Imports System.IO
Imports System.ComponentModel
Imports System.Configuration
Imports System.Globalization


'''<summary>Shared class providing common functions for all pages.</summary>
Public Class Functions

    '''<summary>Constructor.</summary>
    Shared Sub New()
    End Sub


    '''<summary>Returns connection string used throughout the application.</summary>
    Public Shared Function GetConnectionString() As String
        Return ConfigurationSettings.AppSettings("CMSConnectionString")
    End Function


    '''<summary>Creates and returns a new, initialized instance of TreeProvider with current user set.</summary>
    Public Shared Function GetTreeProvider() As TreeProvider
        Dim tree As New TreeProvider

        tree.Connection.ConnectionString = GetConnectionString()
        tree.UserID = GetUserID()
        Return tree
    End Function


    '''<summary>Returns User ID of the current user.</summary>
    Public Shared Function GetUserID() As Integer
        If Web.HttpContext.Current.Session("CMSUserID") Is Nothing And HttpContext.Current.User.Identity.IsAuthenticated Then
            Functions.SetCurrentUserID(Web.HttpContext.Current.User.Identity.Name)
        End If
        If Web.HttpContext.Current.Session("CMSUserID") Is Nothing Then
            Web.HttpContext.Current.Session("CMSUserID") = 0
        End If
        Return CType(Web.HttpContext.Current.Session("CMSUserID"), Integer)
    End Function


    '''<summary>Returns physical path to the files (documents) folder.</summary>
    Public Shared Function GetFilesPath() As String
        Return HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSFilesFolder"))
    End Function


    '''<summary>Returns true if multilingual support is switched on.</summary>
    Public Shared Function UseMultilingualSupport() As Boolean
        If Trim(LCase(ConfigurationSettings.AppSettings("CMSUseMultilingualSupport") & "")) = "true" Then
            Return True
        Else
            Return False
        End If
    End Function


    '''<summary>Removes culture from alias path if you're using multilingual support with hidden culture.</summary>
    Public Shared Function RemoveCultureFromAliasPath(ByVal aliasPath As Object) As String
        Dim alPath As String = CType(aliasPath, String)
        If UseMultilingualSupport() Then
            Return alPath.Substring(alPath.Split("/"c)(1).Length + 1)
        Else
            Return alPath
        End If
    End Function


    '''<summary>Returns Web application virtual path.</summary>
    '''<remarks>It uses the CMSWebApplicationVirtualPath configuration parameter. If it's not used, "~/" is returned.</remarks>
    Public Shared Function GetWebApplicationVirtualPath() As String
        If ConfigurationSettings.AppSettings("CMSWebApplicationVirtualPath") Is Nothing OrElse ConfigurationSettings.AppSettings("CMSWebApplicationVirtualPath") = "" Then
            Return "~/."
        Else
            Return ConfigurationSettings.AppSettings("CMSWebApplicationVirtualPath")
        End If
    End Function


    '''<summary>Returns base disk path of the attachments folder.</summary>
    Public Shared Function GetAttachmentBaseDiskPath() As String
        If Not ConfigurationSettings.AppSettings("CMSAttachmentsFolder") Is Nothing AndAlso ConfigurationSettings.AppSettings("CMSAttachmentsFolder") <> "" Then
            'attachments folder is specified
            If ConfigurationSettings.AppSettings("CMSAttachmentsFolder").StartsWith("/") Then
                'it's a virtual folder
                Return HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSAttachmentsFolder"))
            Else
                'it's a physical path
                Return ConfigurationSettings.AppSettings("CMSAttachmentsFolder")
            End If
        Else
            'attachments folder is not specified - use web path instead
            If GetWebApplicationVirtualPath() <> "" Then
                Return HttpContext.Current.Server.MapPath(GetWebApplicationVirtualPath)
            Else
                Return HttpContext.Current.Server.MapPath("~/.")
            End If
        End If
    End Function


    '''<summary>Returns folder path of the file specified by alias path.</summary>
    '''<param name="aliasPath">Alias Path of the node.</param>
    Public Shared Function GetFileFolderPath(ByVal aliasPath As String) As String
        Dim result As String = ""

        aliasPath = Functions.RemoveCultureFromAliasPath(aliasPath)
        If aliasPath.ToLower.StartsWith("/files") Then
            'this is a files folder
            If (Not ConfigurationSettings.AppSettings("CMSFilesFolder") Is Nothing) AndAlso (ConfigurationSettings.AppSettings("CMSFilesFolder") <> "") Then
                result = Web.HttpContext.Current.Server.MapPath(ConfigurationSettings.AppSettings("CMSFilesFolder"))
            Else
                result = Web.HttpContext.Current.Server.MapPath("~/files")
            End If
            'replace "/files"
            aliasPath = aliasPath.Substring("/files".Length)
        Else
            result = GetAttachmentBaseDiskPath()
        End If
        If result.EndsWith("\") Then
            result = result.Substring(0, result.Length - 1)
        End If
        'remove last part of the alias path (the file name)
        Dim pathSplit As String() = aliasPath.Split("/"c)
        aliasPath = aliasPath.Substring(0, aliasPath.Length - pathSplit(pathSplit.Length - 1).Length)
        If aliasPath.EndsWith("/") Then
            aliasPath = aliasPath.Substring(0, aliasPath.Length - 1)
        End If
        'add alias path
        result &= aliasPath.Replace("/"c, "\"c)
        Return result
    End Function


    '''<summary>Returns true if the current user is authorized to access the given resource (module) with required permission.</summary>
    '''<param name="resourceName">Resource name.</param>
    '''<param name="permissionName">Permission name.</param>
    Public Shared Function IsAuthorizedPerResource(ByVal resourceName As String, ByVal permissionName As String) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return treeSec.IsAuthorizedPerResource(resourceName, permissionName)
    End Function


    '''<summary>Returns true if the current user is authorized to access the given node (document) with required permission.</summary>
    '''<param name="path">Node path.</param>
    '''<param name="pathType">Type of the provided path.</param>
    '''<param name="permission">Required permission.</param>
    '''<remarks>This method checks not only the permissions for the given node. Rather, it also checks if the user was granted with specified permission for the CMS.Content resource.</remarks>
    Public Shared Function IsAuthorizedPerNode(ByVal path As String, ByVal pathType As TreePathTypeEnum, ByVal permission As NodePermissionsEnum) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return Functions.IsAuthorizedPerResource("CMS.Content", treeSec.GetPermissionName(permission)) Or treeSec.IsAuthorizedPerNode(path, pathType, permission)
    End Function


    '''<summary>Returns true if the current user is authorized to access the given class (document type) with required permission.</summary>
    '''<param name="className">Class name in format application.class.</param>    
    '''<param name="permissionName">Name of the required permission..</param>    
    Public Shared Function IsAuthorizedPerClass(ByVal className As String, ByVal permissionName As String) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return treeSec.IsAuthorizedPerClass(className, permissionName)
    End Function


    '''<summary>Sets current user's ID into a session variable for further use.</summary>
    '''<param name="userName">User name.</param>
    Public Shared Sub SetCurrentUserID(ByVal userName As String)
        Dim cn As New GeneralConnection(Functions.GetConnectionString)
        Dim params(0, 2) As Object
        Dim ds As DataSet

        params(0, 0) = "@UserName"
        params(0, 1) = userName
        ds = cn.ExecuteQuery("CMS.User.selectbyusername", params)
        If ds.Tables(0).Rows.Count > 0 Then
            Web.HttpContext.Current.Session("CMSUserID") = ds.Tables(0).Rows(0)("UserID")
        Else
            Web.HttpContext.Current.Session("CMSUserID") = 0
        End If
    End Sub


    '''<summary>Redirects user to the "Access Denied" page.</summary>
    '''<param name="resourceName">Name of the resource that cannot be accessed.</param>
    '''<param name="permissionName">Name of the permission that is not allowed.</param>
    Public Shared Sub RedirectToAccessDenied(ByVal resourceName As String, ByVal permissionName As String)
        Web.HttpContext.Current.Response.Redirect("~/cmsdesk/accessdenied.aspx?resource=" & resourceName & "&permission=" & permissionName)
    End Sub


    '''<summary>Returns preferred UI culture of the current user.</summary>
    Public Shared Function GetPreferredUICulture() As CultureInfo
        If Functions.GetUserID = 0 Then
            Return System.Threading.Thread.CurrentThread.CurrentCulture
        End If
        If HttpContext.Current.Session("CMSUserPreferredUICulture") Is Nothing Then
            'get preferred language
            Dim userObj As DataClass
            Dim tree As TreeProvider = Functions.GetTreeProvider

            userObj = New DataClass("cms.user", Functions.GetUserID, tree.Connection)
            If Trim(CType(userObj.GetValue("PreferredUICultureCode"), String) & "") <> "" Then
                HttpContext.Current.Session("CMSUserPreferredUICulture") = New CultureInfo(Trim(CType(userObj.GetValue("PreferredUICultureCode"), String) & ""))
            Else
                HttpContext.Current.Session("CMSUserPreferredUICulture") = System.Threading.Thread.CurrentThread.CurrentCulture
            End If
        End If
        System.Threading.Thread.CurrentThread.CurrentCulture = CType(HttpContext.Current.Session("CMSUserPreferredUICulture"), CultureInfo)
        System.Threading.Thread.CurrentThread.CurrentUICulture = CType(HttpContext.Current.Session("CMSUserPreferredUICulture"), CultureInfo)
        Return CType(HttpContext.Current.Session("CMSUserPreferredUICulture"), CultureInfo)
    End Function


    '''<summary>Returns current alias path based on base alias path setting and "aliaspath" querystring parameter.</summary>
    Public Shared Function GetAliasPath() As String
        Return ConfigurationSettings.AppSettings("CMSWebSiteBaseAliasPath") & HttpContext.Current.Request.QueryString("aliaspath")
    End Function


    '''<summary>Returns page title tag (applicable only on page templates).</summary>
    '''<param name="defaultAliasPath">Default alias path that is used when no aliaspath parameter is provided in the querystring.</param>
    Public Shared Function GetPageTitleTag(ByVal defaultAliasPath As String) As String
        Dim title As String = ConfigurationSettings.AppSettings("CMSPageTitlePrefix")
        Dim dr As DataRow

        If HttpContext.Current.Request.QueryString("aliaspath") Is Nothing OrElse HttpContext.Current.Request.QueryString("aliaspath") = "" Then
            'use provided alias path only if no alias path is passed in the querystring
            dr = GetPageInfoDR(defaultAliasPath)
        Else
            'use alias path from the querystring
            dr = GetPageInfoDR(Functions.GetAliasPath)
        End If
        If Not dr Is Nothing AndAlso Not dr("MenuItemPageTitle") Is DBNull.Value Then
            title &= CType(dr("MenuItemPageTitle"), String)
        Else
            title &= ""
        End If
        Return "<title>" & HttpContext.Current.Server.HtmlEncode(title) & "</title>"
    End Function


    '''<summary>Returns page meta tag "description" (applicable only on page templates).</summary>
    '''<param name="defaultAliasPath">Default alias path that is used when no aliaspath parameter is provided in the querystring.</param>
    Public Shared Function GetPageMetaDescriptionTag(ByVal defaultAliasPath As String) As String
        Dim description As String
        Dim dr As DataRow

        If HttpContext.Current.Request.QueryString("aliaspath") Is Nothing OrElse HttpContext.Current.Request.QueryString("aliaspath") = "" Then
            'use provided alias path only if no alias path is passed in the querystring
            dr = GetPageInfoDR(defaultAliasPath)
        Else
            'use alias path from the querystring
            dr = GetPageInfoDR(Functions.GetAliasPath)
        End If
        If Not dr Is Nothing AndAlso Not dr("MenuItemPageDescription") Is DBNull.Value Then
            description = CType(dr("MenuItemPageDescription"), String)
        Else
            description = ""
        End If
        Return "<meta name=""description"" content=""" & HttpContext.Current.Server.HtmlEncode(description) & """/>"
    End Function


    '''<summary>Returns page meta tag "keywords" (applicable only on page templates).</summary>
    '''<param name="defaultAliasPath">Default alias path that is used when no aliaspath parameter is provided in the querystring.</param>    
    Public Shared Function GetPageMetaKeyWordsTag(ByVal defaultAliasPath As String) As String
        Dim keywords As String
        Dim dr As DataRow

        If HttpContext.Current.Request.QueryString("aliaspath") Is Nothing OrElse HttpContext.Current.Request.QueryString("aliaspath") = "" Then
            'use provided alias path only if no alias path is passed in the querystring
            dr = GetPageInfoDR(defaultAliasPath)
        Else
            'use alias path from the querystring
            dr = GetPageInfoDR(Functions.GetAliasPath)
        End If
        If Not dr Is Nothing AndAlso Not dr("MenuItemPageKeyWords") Is DBNull.Value Then
            keywords = CType(dr("MenuItemPageKeyWords"), String)
        Else
            keywords = ""
        End If
        Return "<meta name=""keywords"" content=""" & HttpContext.Current.Server.HtmlEncode(keywords) & """/>"
    End Function


    '''<summary>Returns CSS file link tag.</summary>
    '''<param name="defaultAliasPath">Default alias path that is used when no aliaspath parameter is provided in the querystring.</param>        
    Public Shared Function GetCSSFileLink(ByVal defaultAliasPath As String) As String
        Dim dr As DataRow
        Dim result As String
        Dim appPath As String
        Dim cssfile As String
        If ConfigurationSettings.AppSettings("CMSCSSVirtualPath") Is Nothing Then
            appPath = HttpContext.Current.Request.ApplicationPath
            If Not appPath.EndsWith("/") Then
                appPath &= "/"
            End If
            cssfile = appPath & "main.css"
        Else
            cssfile = ConfigurationSettings.AppSettings("CMSCSSVirtualPath")
        End If

        result = "<link href=""" & HttpContext.Current.Server.HtmlEncode(cssfile) & """ type=""text/css"" rel=""stylesheet"">" & vbCrLf
        Return result
    End Function


    '''<summary>Returns page title tag, meta tag "description" and meta tag "keywords" (applicable only on page templates).</summary>
    '''<param name="defaultAliasPath">Default alias path that is used when no aliaspath parameter is provided in the querystring.</param>        
    Public Shared Function GetPageTags(ByVal defaultAliasPath As String) As String
        Return Functions.GetPageTitleTag(defaultAliasPath) & vbCrLf & Functions.GetPageMetaKeyWordsTag(defaultAliasPath) & vbCrLf & Functions.GetPageMetaDescriptionTag(defaultAliasPath) & vbCrLf & Functions.GetCSSFileLink(defaultAliasPath)
    End Function


    '''<summary>Returns DataRow containing page template information.</summary>
    '''<param name="aliasPath">Current alias path.</param>
    Public Shared Function GetPageInfoDR(ByVal aliasPath As String) As DataRow
        Dim params(0, 2) As Object
        Dim ds As DataSet
        Dim cn As GeneralConnection
        Dim cacheMinutes As Integer
        Dim dr As DataRow

        'get the full path if multilingual support is used
        If (ConfigurationSettings.AppSettings("CMSUseMultilingualSupport") & "").ToLower.Trim = "true" Then
            If aliasPath = "/" Then
                aliasPath = "/" & GetPreferredCulture()
            Else
                aliasPath = "/" & GetPreferredCulture() & aliasPath
            End If
        End If
        'try to find value in cache
        If HttpContext.Current.Cache.Item("cmspageinfo_" & aliasPath.ToLower) Is Nothing Then
            'read data from database
            cn = New GeneralConnection(GetConnectionString)
            params(0, 0) = "@AliasPath"
            params(0, 1) = ConfigurationSettings.AppSettings("CMSWebSiteBaseAliasPath") & aliasPath
            ds = cn.ExecuteQuery("cms.menuitem.selectmenuitemfromaliaspath", params)
            If ds.Tables(0).Rows.Count > 0 Then
                If Not ConfigurationSettings.AppSettings("CMSCachePageInfoMinutes") Is Nothing Then
                    cacheMinutes = CType(ConfigurationSettings.AppSettings("CMSCachePageInfoMinutes"), Integer)
                Else
                    cacheMinutes = 0
                End If
                HttpContext.Current.Cache.Insert("cmspageinfo_" & aliasPath.ToLower, ds.Tables(0).Rows(0), Nothing, Now.AddMinutes(cacheMinutes), System.Web.Caching.Cache.NoSlidingExpiration)
            End If
            If ds.Tables(0).Rows.Count > 0 Then
                dr = ds.Tables(0).Rows(0)
            End If
        Else
            dr = CType(HttpContext.Current.Cache.Item("cmspageinfo_" & aliasPath.ToLower), DataRow)
        End If
        Return dr
    End Function


    '''<summary>Returns provided path without culture code.</summary>
    '''<param name="path">Path to be modified.</param>
    Public Shared Function RemoveCultureFromPath(ByVal path As String) As String
        Dim result As String
        Dim pth As String = CType(path, String)
        If Trim(LCase(ConfigurationSettings.AppSettings("CMSUseMultilingualSupport") & "")) = "true" Then
            result = pth.Substring(pth.Split("/"c)(1).Length + 1)
        Else
            result = pth
        End If
        Return result
    End Function


    '''<summary>Returns preferred culture code (as string). You can modify this function so that it determines the preferred culture using some other algorithm.</summary>
    Public Shared Function GetPreferredCulture() As String
        If HttpContext.Current.Session("CMSPreferredCulture") Is Nothing Then
            HttpContext.Current.Session("CMSPreferredCulture") = ConfigurationSettings.AppSettings("CMSDefaultCultureCode")
        End If
        Return CType(HttpContext.Current.Session("CMSPreferredCulture"), String)
    End Function


    '''<summary>Returns type (such as "cms.article") of the current document.</summary>
    Public Shared Function GetDocumentType() As String
        If Trim(HttpContext.Current.Request.QueryString("aliaspath") & "") = "" Then
            Return ""
        Else
            Return GetDocumentType(Trim(HttpContext.Current.Request.QueryString("aliaspath") & ""))
        End If
    End Function


    '''<summary>Returns type (such as "cms.article") of the specified document.</summary>
    '''<param name="aliasPath">Alias path of the document.</param>
    Public Shared Function GetDocumentType(ByVal aliasPath As String) As String
        Dim tree As TreeProvider
        Dim node As TreeNode

        If Trim(aliasPath & "") = "" Then
            Return GetDocumentType("/")
        Else
            'get the full path if multilingual support is used
            If (ConfigurationSettings.AppSettings("CMSUseMultilingualSupport") & "").ToLower.Trim = "true" Then
                If aliasPath = "/" Then
                    aliasPath = "/" & GetPreferredCulture()
                Else
                    aliasPath = "/" & GetPreferredCulture() & aliasPath
                End If
            End If

            tree = GetTreeProvider()
            node = tree.SelectSingleNode(aliasPath, TreePathTypeEnum.AliasPath)
            If node Is Nothing Then
                Return ""
            Else
                Return node.ClassName
            End If
        End If
    End Function


    '''<summary>Returns node representing the current page (the nearest parent page).</summary>
    Public Shared Function GetCurrentPage() As TreeNode
        Dim dr As DataRow = Functions.GetPageInfoDR(Functions.GetAliasPath)
        Return Functions.GetTreeProvider.SelectSingleNode(CType(dr("AliasPath"), String), TreePathTypeEnum.AliasPath, "cms.menuitem")
    End Function


    '''<summary>Returns node representing the current document.</summary>
    Public Shared Function GetCurrentDocument() As TreeNode
        Dim aliasPath As String = Functions.GetAliasPath
        If aliasPath Is Nothing OrElse aliasPath = "" Then
            aliasPath = "/"
        End If

        'get the full path if multilingual support is used
        If (ConfigurationSettings.AppSettings("CMSUseMultilingualSupport") & "").ToLower.Trim = "true" Then
            If aliasPath = "/" Then
                aliasPath = "/" & GetPreferredCulture()
            Else
                aliasPath = "/" & GetPreferredCulture() & aliasPath
            End If
        End If

        'get the node
        Return Functions.GetTreeProvider.SelectSingleNode(aliasPath, TreePathTypeEnum.AliasPath)
    End Function


    '''<summary>Returns true if the current user is member of the given role.</summary>
    '''<param name="roleName">Role name.</param>
    Public Shared Function IsInRole(ByVal roleName As String) As Boolean
        Dim treeSec As New TreeSecurityProvider(Functions.GetTreeProvider)
        Return treeSec.IsInRole(roleName, Functions.GetUserID)
    End Function


    '''<summary>Writes event to the event log.</summary>
    '''<param name="eventType">Type of the event. I = information, E = error, W = warning</param>
    '''<param name="source">Source of the event (Content, Administration, etc.)</param>
    '''<param name="eventCode">Event code (Security, Update, Delete, etc.)</param>
    '''<param name="nodeId">ID value of the document.</param>
    '''<param name="nodeNamePath">NamePath value of the document.</param>
    '''<param name="eventDescription">Detailed description of the event.</param>
    Public Shared Sub LogEvent(ByVal eventType As String, ByVal source As String, ByVal eventCode As String, ByVal nodeId As Integer, ByVal nodeNamePath As String, ByVal eventDescription As String)
        Dim tree As TreeProvider = New TreeProvider(0, New GeneralConnection(Functions.GetConnectionString))
        Dim log As New Kentico.CMS.EventLog.EventLogProvider(tree.Connection)
        log.LogEvent(eventType, Now, source, eventCode, 0, HttpContext.Current.User.Identity.Name, nodeId, nodeNamePath, HttpContext.Current.Request.UserHostAddress, eventDescription)
    End Sub


    '''<summary>Adds the postfix of the generated URLs according to the CMSFriendlyURLStyle web.config parameter value.</summary>
    '''<param name="path">Path to which the extension should be added.</param>
    Public Shared Function AddUrlExtension(ByVal path As String) As String
        If (Not ConfigurationSettings.AppSettings("CMSFriendlyURLStyle") Is Nothing) AndAlso ConfigurationSettings.AppSettings("CMSFriendlyURLStyle").Trim.ToLower <> "" Then
            If path = "/" Or path = "~" Or path = "~/" Or path = "~/." Then
                Return path
            Else
                Return path & ConfigurationSettings.AppSettings("CMSFriendlyURLStyle").Trim.ToLower
            End If
        Else
            Return path
        End If
    End Function


    '''<summary>Returns URL of the document specified by alias path.</summary>
    '''<param name="aliasPath">Alias path of the document.</param>
    Public Shared Function GetUrl(ByVal aliasPath As Object) As String
        Dim url As String
        If aliasPath Is Nothing Then
            url = "/"
        Else
            url = CType(aliasPath, String)
        End If
        url = Functions.RemoveCultureFromAliasPath(url)
        url = Functions.AddUrlExtension(url)
        If HttpContext.Current.Request.ApplicationPath.EndsWith("/") Then
            url = HttpContext.Current.Request.ApplicationPath.Substring(0, HttpContext.Current.Request.ApplicationPath.Length - 1) & url
        Else
            url = HttpContext.Current.Request.ApplicationPath & url
        End If
        Return HttpContext.Current.Server.UrlPathEncode(url)
    End Function


    '''<summary>Returns URL of the file document specified by alias path.</summary>
    '''<param name="aliasPath">Alias path of the file document.</param>
    Public Shared Function GetFileUrl(ByVal aliasPath As Object) As String
        Dim url As String = "getfile.aspx"
        If HttpContext.Current.Request.ApplicationPath.EndsWith("/") Then
            url = HttpContext.Current.Request.ApplicationPath & url
        Else
            url = HttpContext.Current.Request.ApplicationPath & "/" & url
        End If
        url = url & "?aliaspath=" & Functions.RemoveCultureFromAliasPath(aliasPath)
        Return HttpContext.Current.Server.UrlPathEncode(url)
    End Function


    '''<summary>Returns link to the file with specified virtual path.</summary>
    '''<param name="filePath">Virtual path of the file, such as /products/nokia/nokia_large.gif.</param>
    Public Shared Function GetUrlForFilePath(ByVal filePath As Object) As String
        Dim url As String = "getfile.aspx"
        If HttpContext.Current.Request.ApplicationPath.EndsWith("/") Then
            url = HttpContext.Current.Request.ApplicationPath & url
        Else
            url = HttpContext.Current.Request.ApplicationPath & "/" & url
        End If
        url = url & "?filepath=" & filePath.ToString
        Return HttpContext.Current.Server.UrlPathEncode(url)
    End Function


    '''<summary>Returns first N levels of the given alias path.</summary>
    '''<param name="aliasPath">Alias path.</param>
    '''<param name="level">Number of levels to be returned.</param>
    Public Shared Function GetPathLevel(ByVal aliasPath As String, ByVal level As Integer) As String
        Dim resultPath As String = ""
        Dim i As Integer
        Dim splitPath() As String

        If aliasPath Is Nothing Then
            Return ""
        End If
        splitPath = aliasPath.Split("/"c)
        For i = 1 To level
            resultPath &= "/" & splitPath(i)
        Next
        Return resultPath
    End Function

    '''<summary>Encodes URL(just redirection for use with aspx code</summary>
    '''<param name="url">URL to encode</param>
    Public Shared Function UrlPathEncode(ByVal url As Object) As String
        Dim path As String
        If Not url Is Nothing Then
            path = CType(url, String)
        Else
            path = ""
        End If
        Return HttpContext.Current.Server.UrlPathEncode(path)
    End Function

End Class
